package fr.uca.iut.clfreville2.gui;

import fr.uca.iut.clfreville2.gui.image.ImageSupplier;
import fr.uca.iut.clfreville2.gui.image.StandardImageSupplier;
import fr.uca.iut.clfreville2.gui.list.NameableListCell;
import fr.uca.iut.clfreville2.gui.thread.Ticker;
import fr.uca.iut.clfreville2.model.SensorRegistry;
import fr.uca.iut.clfreville2.model.binding.ToBooleanBinding;
import fr.uca.iut.clfreville2.model.sensor.ManualSensor;
import fr.uca.iut.clfreville2.model.sensor.Sensor;
import fr.uca.iut.clfreville2.persistence.StubSensorRegistryLoader;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ListView;
import javafx.scene.control.Slider;
import javafx.scene.control.Spinner;
import javafx.scene.control.TextField;
import javafx.scene.image.ImageView;
import javafx.scene.text.Text;
import javafx.stage.Stage;

public class MainWindows {

    private final SensorRegistry registry = new StubSensorRegistryLoader().load();
    private final ImageSupplier imageSupplier = new StandardImageSupplier();
    private final Ticker ticker;
    private final ModalFactory modalFactory;

    @FXML
    private ListView<Sensor> sensorsList;

    @FXML
    private Text sensorId;

    @FXML
    public TextField sensorName;

    @FXML
    private Button changeBtn;

    @FXML
    private Button visualizeBtn;

    @FXML
    private Spinner<Integer> updateInterval;

    @FXML
    private CheckBox autoUpdate;

    public MainWindows(Stage primaryStage) {
        this.ticker = new Ticker(registry);
        this.modalFactory = new ModalFactory(primaryStage);
        primaryStage.setOnHidden((e) -> ticker.interrupt());
        ticker.start();
    }

    @FXML
    public void onChangeClick() {
        Sensor selected = getSelectedSensor();
        if (!(selected instanceof ManualSensor sensor)) {
            return;
        }
        modalFactory.createModal(selected, root -> {
            Slider slider = new Slider(-5, 30, sensor.getTemperature());
            slider.setBlockIncrement(0.5);
            slider.setMajorTickUnit(0.5);
            slider.setMinorTickCount(0);
            slider.setSnapToTicks(true);
            slider.valueProperty().bindBidirectional(sensor.temperatureProperty());
            root.getChildren().add(slider);
        }).show();
    }

    @FXML
    public void onVisualizeClick() {
        Sensor selected = getSelectedSensor();
        if (selected == null) {
            return;
        }
        modalFactory.createModal(selected, root -> {
            ImageView imageView = new ImageView();
            imageView.setPreserveRatio(true);
            imageView.setFitHeight(200);
            selected.temperatureProperty().addListener((observable, old, newValue) -> imageView.setImage(imageSupplier.apply(selected)));
            imageView.setImage(imageSupplier.apply(selected));
            root.getChildren().add(imageView);
        }).show();
    }

    @FXML
    private void initialize() {
        bindSensorList();
        bindActiveButtons();
        bindUpdate();
    }

    @FXML
    private void bindSensorList() {
        sensorsList.setCellFactory(list -> new NameableListCell<>());
        sensorsList.itemsProperty().bind(registry.sensorsProperty());
        sensorsList.getSelectionModel().selectedItemProperty().addListener((list, oldValue, newValue) -> {
            if (oldValue != null) {
                sensorName.textProperty().unbindBidirectional(oldValue.nameProperty());
            }
            if (newValue != null) {
                sensorId.textProperty().bind(newValue.displayNameExpression());
                sensorName.textProperty().bindBidirectional(newValue.nameProperty());
            } else {
                sensorId.textProperty().unbind();
                sensorId.setText(null);
            }
        });
    }

    @FXML
    private void bindActiveButtons() {
        changeBtn.visibleProperty().bind(new ToBooleanBinding<>(
                sensorsList.getSelectionModel().selectedItemProperty(),
                treeItem -> treeItem instanceof ManualSensor
        ));
        visualizeBtn.visibleProperty().bind(sensorsList.getSelectionModel().selectedItemProperty().isNotNull());
    }

    @FXML
    private void bindUpdate() {
        updateInterval.getValueFactory().valueProperty().bindBidirectional(ticker.millisPerTickProperty().asObject());
        autoUpdate.selectedProperty().bindBidirectional(ticker.runningProperty());
    }

    private Sensor getSelectedSensor() {
        return sensorsList.getSelectionModel().getSelectedItem();
    }
}
